#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# __openerp__
# ---------------------------------------------------------------------
# Copyright (c) 2017 Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the LICENCE attached (see LICENCE file) in the distribution
# package.
#
# Created on 2017-08-01

{
    "name": "test_timedelta",
    "author": "Merchise Autrement [~º/~] and Contributors",
    "description": "Test for the field.Timedelta extension",
    "depends": ["base"],
    "installable": True,
    "auto_install": False,
}
